@@ -10,7 +10,7 @@ from matplotlib import pyplot as plt
 from scipy.optimize import minimize
 from astropy.table import Table
 
-from laspec.mrs import MrsFits
+from .mrs import MrsFits
 
 SOL_kms = constants.c.value / 1000
 
@@ -850,7 +850,6 @@ def test_lmfit():
 
 def test_new_rvm():
     import joblib
-    from laspec.ccf import RVM
     rvm = RVM(joblib.load("/Users/cham/PycharmProjects/laspec/laspec/data/rvm/v8_rvm_pmod.dump"),
               joblib.load("/Users/cham/PycharmProjects/laspec/laspec/data/rvm/v8_rvm_wave_mod.dump"),
               joblib.load("/Users/cham/PycharmProjects/laspec/laspec/data/rvm/v8_rvm_flux_mod.dump"))
